#include "enna.h"
#include <stdio.h>
#include <stdlib.h>
#include "enna_config.h"
#include "enna_mainmenu.h"
#include "enna_module.h"
#include "enna_module_music.h"
#include "enna_module_video.h"
#include "enna_module_playlist.h"
#include "enna_mediaplayer.h"
#include "enna_miniplayer.h"
#include "enna_lirc.h"
#include "enna_event.h"

struct keyboard_mapping
{
	char *name;
	char *code;
	enna_event evt;
} keyboard_mapping[] = {
	    {"Back",		"BackSpace", 	enna_event_escape	},
	    {"Exit", 		"q", 		enna_event_exit		},
	    {"Pause", 		"p", 		enna_event_pause	},
	    {"FastForward", 	"Next",		enna_event_fastforward	},
	    {"Rewind", 		"Prior",	enna_event_rewind	},
	    {"Stop", 		"s", 		enna_event_stop		},
	    {"Mute", 		"m", 		enna_event_mute		},
	    {"Down", 		"Down", 	enna_event_down		},
	    {"Up", 		"Up", 		enna_event_up		},
	    {"Right", 		"Right", 	enna_event_right	},
	    {"Left", 		"Left", 	enna_event_left		},
	    {"Aspect", 		"a", 		enna_event_aspect	},
	    {"Language", 	"l", 		enna_event_language	},
	    {"FullScreen", 	"f", 		enna_event_fullscreen	},
	    {"AddToPlaylist", 	"KP_Delete",	enna_event_addtopl	},
	    {"Select", 		"Return",	enna_event_enter	},
	    {"Next", 		"b",     	enna_event_next 	},
	    {"Prev", 		"n", 	        enna_event_prev		},
	    {"", 		"",		enna_event_none 	}
};

EAPI void
enna_event_process_event(Enna *enna, enna_event event)
{

   int i = 0;
 
   for (i = 0; i < NBR_ELEMENTS(keyboard_mapping); i++)
     {  
       
       if (keyboard_mapping[i].evt == event)
         {
           dbg("Sent event : %s\n", keyboard_mapping[i].name);
           break;
         }
     }

   /* If event is a media event : play/pause/next/prev/ffw/rew/stop/mute/aspect/language 
    * event is send to mediaplayer object
    * if a video is playing left/right/up/down mediaplayer object has focus set
    * so we can detect this and send event to mediaplayer object 
    * if no video is playing, left/right are used for menu navigation
   */
  switch(event)
    {
    case enna_event_play:
    case enna_event_pause :
    case enna_event_next :
    case enna_event_prev :
    case enna_event_fastforward :
    case enna_event_rewind :
    case enna_event_stop :
    case enna_event_mute :
    case enna_event_aspect :
    case enna_event_language :
      if (enna->fs_obj)
        enna_module_process_event(enna->modules, event);
      else
        enna_mediaplayer_process_event(enna->mediaplayer, event);
      return;
    case enna_event_left :
    case enna_event_right :
      if (enna_mediaplayer_has_focus_get(enna->mediaplayer))
        {
          enna_mediaplayer_process_event(enna->mediaplayer, event);
          return;
        }
      else if (!enna_mainmenu_has_focus_get(enna->mainmenu))
        {
          enna_mainmenu_has_focus_set(enna->mainmenu, 1);
          enna_module_has_focus_set(enna->modules, 0);
        }
      break;
    case enna_event_up :
    case enna_event_down :
      if (enna_mediaplayer_has_focus_get(enna->mediaplayer))
        {
          enna_mainmenu_has_focus_get(enna->mediaplayer);
          return;
        }
      if (!enna_module_has_focus_get(enna->modules))
        {
          enna_module_has_focus_set(enna->modules, 1);
          enna_mainmenu_has_focus_set(enna->mainmenu, 0);
        }
      break;
    case enna_event_exit :
      if (enna_mediaplayer_has_focus_get(enna->mediaplayer))
        {
          enna_mediaplayer_process_event(enna->mediaplayer, event);
          return;
        }
      else
        ecore_main_loop_quit();
      return;
    case enna_event_fullscreen :
      enna_togle_fullscreen(enna);
    default:
      break;
    }

  if (enna_mainmenu_has_focus_get(enna->mainmenu))
	  enna_mainmenu_process_event(enna->mainmenu, event);
  else if (enna_module_has_focus_get(enna->modules))
	  enna_module_process_event(enna->modules, event);
  else
	  dbg("No object has focus ! ERROR\n");
}


EAPI void
enna_event_lirc_code(void *data, char *code)
{
  Enna* enna;

  enna_event event = enna_event_none;
  int i = 0;
  enna = (Enna*) data;
  
  if (!enna)
    return;

  while (keyboard_mapping[i].evt != enna_event_none)
      {
	  if (!strcmp(code, keyboard_mapping[i].code))
	      {
		  event = keyboard_mapping[i].evt;
		  break;
	      }
	  i++;
      }

  enna_event_process_event(enna, event);
}


/* key down callback */
EAPI void
enna_event_bg_key_down_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
  Enna* enna;
  Evas_Event_Key_Down *ev;
  enna_event event = enna_event_none;
  int i = 0;
  enna = (Enna*) data;
  ev = (Evas_Event_Key_Down *)event_info;

  if (!ev || ! enna)
    return ;
  /* if left or right pressed and if mainemnu doesn't have focus
   * set focus to mainemenu */
  /* if up down esc pressed and if moduled doesn't have focus
  * set focus to modules */


  while (keyboard_mapping[i].evt != enna_event_none)
      {
	  if (!strcmp(ev->keyname, keyboard_mapping[i].code))
	      {
		  event = keyboard_mapping[i].evt;
		  break;
	      }
	  i++;
      }

  enna_event_process_event(enna, event);
}

EAPI void enna_togle_fullscreen(Enna *enna)
{
	if (ecore_evas_fullscreen_get(enna->ee))
		ecore_evas_fullscreen_set(enna->ee, 0);
	else
		ecore_evas_fullscreen_set(enna->ee, 1);
}

EAPI void enna_event_init(Enna *enna)
{
	int i = 0;
	while (keyboard_mapping[i].evt != enna_event_none)
	    {
		char *value = enna_config_get_conf_value("keyboard", keyboard_mapping[i].name);
		if (value)
			keyboard_mapping[i].code = strdup(value);
		i++;
	    }
};
