/****************************************************************************
 ** enna_lirc.c *************************************************************
 ****************************************************************************
 *
 * enna_lirc  - adds lirc support to ennas
 *
 * Copyright (C) 2007 Philippe Bridant <philippe.bridant@smartjog.com>
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <Emotion.h>
#include <Eet.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>





#include "enna.h"
#include "enna_config.h"
#include "enna_socket.h"
#include "enna_lirc.h"
#include "enna_event.h"
#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DEFAULT_LIRC_PORT "8765"
#define DEFAULT_LIRC_SERVER "127.0.0.1"
#define LIRC_CONF_FILE "/etc/lirc/enna.conf"
#define DEFAULT_LIRC_CONF_FILE "enna.conf"
#define LIRC_CONF_SECTION_NAME "enna"
#define BUF_SIZE	4096

#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))

typedef struct _E_Smart_Data E_Smart_Data;

#ifndef HAVE_LIRC_CLIENT

struct key_value
{
	char *key;
	char *value;
};

struct conf_entry
{
	char 			*prog;
	char 			*button;
	char 			*config;
	int 			repeat;
	int			delay;
	enna_event 	        evt;
};

struct _E_Smart_Data
{
	int			sd;
	int			nb_conf;
	struct conf_entry	*conf;
	Ecore_Fd_Handler 	*handler;
	Enna 			*enna;
};

struct enna_lirc_codes_to_enna_events
{
	char			*code_name;
	enna_event 	        event;
};

static struct enna_lirc_codes_to_enna_events enna_lirc_code_relations[] =  {
	    {"Up", 		enna_event_up},
	    {"Down", 		enna_event_down},
	    {"Left", 		enna_event_left},
	    {"Right", 		enna_event_right},
	    {"Stop", 		enna_event_stop},
	    {"Play", 		enna_event_play},
	    {"Next", 		enna_event_next},
	    {"Prev", 		enna_event_prev},
	    {"Pause", 		enna_event_pause},
	    {"FastForward", 	enna_event_fastforward},
	    {"Rewind", 		enna_event_rewind},
	    {"Select", 		enna_event_enter},
	    {"Exit", 		enna_event_exit},
	    {"Escape", 		enna_event_escape},
	    {"AspectRatio", 	enna_event_aspect},
	    {"Language", 	enna_event_language},
	    {"FullScreen", 	enna_event_fullscreen},
	    {"Mute", 		enna_event_mute},
	    {NULL,		enna_event_none}
};


static void enna_lirc_add_action(E_Smart_Data *sd, struct conf_entry *entry)
{
	// First ignore any conf that is not for enna
	if (strcmp(entry->prog, "enna"))
		return;
	if (!entry->button)
		printf("enna_lirc_add_action: No button assigned to lirc conf entry\n"), exit(0);
	if (!entry->config)
		printf("enna_lirc_add_action: No config assigned to lirc conf entry\n"), exit(0);


	// Then we look for the action in our registered actions
	int i = 0;
	entry->evt = enna_event_none;
	while (enna_lirc_code_relations[i].code_name)
	    {
		if (!strcmp(enna_lirc_code_relations[i].code_name, entry->config))
		    {
			entry->evt = enna_lirc_code_relations[i].event;
			break;
		    }
		i++;
	    }

	if (entry->evt == enna_event_none)
		printf("Cannot find action '%s'\n", entry->config), exit(0);

	// Resize our action table
	sd->conf = realloc(sd->conf, (sd->nb_conf+1) * sizeof(*(sd->conf)));
	if (!sd->conf)
	    {
		printf("enna_lirc_add_action: Cannot allocate memory\n");
		exit(0);
	    }

	// And save the new conf
	sd->conf[sd->nb_conf].prog 	= NULL;// No need to keep this field, must be "enna"
	sd->conf[sd->nb_conf].button 	= strdup(entry->button);
	sd->conf[sd->nb_conf].config 	= strdup(entry->config);
	sd->conf[sd->nb_conf].repeat 	= entry->repeat;
	sd->conf[sd->nb_conf].delay 	= entry->delay;
	sd->conf[sd->nb_conf].evt	= entry->evt;
	sd->nb_conf++;
}

static void enna_read_key_value(char *buffer, char *end_of_line, struct key_value *kv)
{
	kv->key = NULL;
	kv->value = NULL;

	// Removing leading spaces
	while (*buffer == ' ')
		buffer++;

	if (*buffer == '#') // Comment
		return;

	if (*buffer == '\0') // Empty line
		return;

	// Removing end spaces
	while (end_of_line != buffer && *end_of_line == ' ')
	    {
		*end_of_line = '\0';
		end_of_line--;
	    }

	char *equal = strchr(buffer, '=');
	char *value = equal + 1;
	kv->key = buffer;

	// No '=', so no value
	if (!equal)
		return;

	// If there is a '=' it can be useless ' '
	*equal = ' ';
	while (equal != buffer && *equal == ' ')
	    {
		*equal = '\0';
		equal--;
	    }

	// Removing ' ' before value
	while (*value == ' ')
	    {
		*value = '\0';
		value++;
	    }
	kv->value = value;
}

static void enna_parse_lirc_conf(E_Smart_Data *sd, char *buffer)
{
	int action_begun = 0;
	char *next_line, *end_of_line;
	struct conf_entry entry;
	struct key_value kv;

	memset(&entry, 0, sizeof(entry));

	do
	    {
		end_of_line = NULL;
		// Getting the next line to be proceed
		next_line = strchr(buffer, '\n');
		if (next_line)
		    {
			end_of_line = next_line - 1;
			*next_line = '\0';
			next_line++;
		    }
		kv.key=NULL;
		kv.value=NULL;
		// Let's parse this new line
		enna_read_key_value(buffer, end_of_line, &kv);

		// Line was empty
		if (!kv.key && !kv.value)
		    {
			buffer = next_line;
			continue;
		    }

		// We are between actions
		if (!action_begun)
		    {
			// and we found 'begin'
			if (!strcmp(kv.key, "begin"))
				action_begun = 1;
			else
				printf("Unknown keyword : %s\n", kv.key), exit(0);
		    }
		else
		    {
			// If we have not key=value to we must have 'end'
			if (!kv.value)
			    {
				if (!strcmp(kv.key, "end"))
				    {
					action_begun = 0;
					// Saving action
					enna_lirc_add_action(sd, &entry);
				    }
				else
					printf("Unknown keyword : %s\n", buffer), exit(0);
			    }
			else
			    {
				// Saving key=value in our temporaty structure
				if (!strcmp(kv.key, "prog"))
					entry.prog = kv.value;
				else if (!strcmp(kv.key, "button"))
					entry.button = kv.value;
				else if (!strcmp(kv.key, "delay"))
					entry.delay = atoi(kv.value);
				else if (!strcmp(kv.key, "repeat"))
					entry.repeat = atoi(kv.value);
				else if (!strcmp(kv.key, "config"))
					entry.config = kv.value;
				else
					printf("Unknown key : %s\n", kv.key), exit(0);
			    }
		    }

		// Let's parse the next line
		buffer = next_line;
	    }
	while (buffer);
}

static void enna_read_lirc_conf(E_Smart_Data *sd, char *filename)
{
	int           fd;
	struct stat   st;

	if (stat(filename, &st))
	    {
		printf("Cannot stat file %s\n", filename);
		return;
	    }
	char *conffile = malloc(st.st_size + 1);
	if (!conffile)
	    {
		printf("Cannot malloc %d bytes\n", (int)st.st_size);
		return;
	    }

	if ((fd = open(filename, O_RDONLY)) < 0)
	    {
		printf("Cannot open file\n");
		return;
	    }

	int ret = read(fd, conffile, st.st_size);
	if (ret != st.st_size)
	    {
		printf("Cannot read lirc conf file entirely, read only %d bytes\n", ret);
		return;
	    }
	conffile[ret] = '\0';
	enna_parse_lirc_conf(sd, conffile);
}

static void enna_find_code(E_Smart_Data *sd, char *line)
{
	int i;

	// Extract code from the line that lirc server gave to us
	// example: 0x43214512351 02 NextChapter
	char *repeat = strchr(line, ' ');
	if (!repeat)
		return;
	*repeat = '\0';
	repeat++;

	char *code = strchr(repeat, ' ');
	if (!code)
		return;
	*code = '\0';
	code++;

	char *remote = strchr(code, ' ');
	if (!remote)
		return;
	*remote = '\0';
	remote++;

	*(remote-1) = '\0';
	for (i = 0; i < sd->nb_conf; i++)
		if (!strcmp(sd->conf[i].button, code))
			enna_event_process_event(sd->enna, sd->conf[i].evt);
}

static int enna_find_lirc_line(E_Smart_Data *sd, char *buf, int *n)
{
	char *end_buf;
	char line[BUF_SIZE];

	if ((end_buf = strchr(buf, '\n')))
	    {
		memcpy(line, buf, end_buf - buf);
		line[end_buf - buf] = 0;
		enna_find_code(sd, line);
		*n -= (end_buf - buf) + 1;
		memcpy(buf, end_buf + 1, *n);
		buf[*n] = 0;
		return 1;
	    }
	return 0;
}

static int enna_read_lirccode(void *data, Ecore_Fd_Handler *fd_handler)
{
	E_Smart_Data 	*sd = (E_Smart_Data*)data;
	static char buf[BUF_SIZE];
	static int  n = 0;
	int ret;

	while ((ret = read(sd->sd, &buf[n], BUF_SIZE-n)) > 0)
	    {
		n += ret;
		buf[n] = 0;
		while (enna_find_lirc_line(sd, buf, &n));
	    }
	return 1;
}

static void init_lirc_conf(E_Smart_Data *sd)
{
	char buf[1024];
	struct passwd *p = getpwuid(getuid());
	if (!p)
	    {
		printf("Cannot get home directory\n");
		exit(0);
	    }

	snprintf(buf, 1024, "%s/%s", p->pw_dir, DEFAULT_LIRC_CONF_FILE);
	enna_read_lirc_conf(sd, enna_config_get_conf_value_or_default("lirc", "lirc_conf", buf));
}

	EAPI void 
enna_init_lirc(Enna *enna)
{
	E_Smart_Data 		*sd;

	sd = calloc(1, sizeof(E_Smart_Data));
	if (!sd)
		return;
	sd->enna = enna;
	sd->nb_conf = 0;
	sd->conf = NULL;

	sd->sd = enna_connect(
			enna_config_get_conf_value_or_default("lirc", "lirc_server", DEFAULT_LIRC_SERVER),
			atoi(enna_config_get_conf_value_or_default("lirc", "lirc_port", DEFAULT_LIRC_PORT)),
			1,
			NULL, 0);
	if (sd->sd < 0)
	    {
		printf("Cannot connect to lirc server\n");
		return;
	    }
	init_lirc_conf(sd);

	sd->handler = ecore_main_fd_handler_add(sd->sd, ECORE_FD_READ, enna_read_lirccode, sd, NULL,  NULL);
}

void	enna_deinit_lirc()
{
}
#endif

#ifdef HAVE_LIRC_CLIENT

/* Lirc Stuff */
int enna_lirc_init(Enna* enna);
int enna_check_lirc_data(void* data);
#include <lirc/lirc_client.h>


/* LIRC client code */
int enna_lirc_init(Enna* enna)
{
  int fd;    
   
  if ( (fd = lirc_init( "enna",1 ) ) == -1 ) 
    {
      dbg("could not initialize LIRC support\n");
      return -1;
    }
   
  if (lirc_readconfig(NULL,&enna->lirc_cfg,NULL) != 0) {
    lirc_deinit();
    dbg("could not find Lirc config file\n");
  }

  fcntl(fd,F_SETFL,O_NONBLOCK);
  fcntl(fd,F_SETFD,FD_CLOEXEC);
   
  enna->lirc_timer = ecore_timer_add(0.01, enna_check_lirc_data, enna); 
  enna->lirc_fd = fd;
  return fd;
}

int enna_check_lirc_data(void* data)
{
  Enna* enna = (Enna*) data;
   
  char *code,*event;
  int ret=-1;
    
  //strcpy(event,"");
   
  while (lirc_nextcode(&code)==0  &&  code!=NULL) 
    {
      while((ret = lirc_code2char(enna->lirc_cfg,code,&event)) == 0 && event!=NULL)
	{
        
	  dbg("key pressed = %s\n", event);
	  enna_event_lirc_code((void*)enna, event);
         
	}
    }
  free(code);
   
  return 1;
}
#endif
