/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Ecore_Evas.h>
#include <Ecore_Str.h>
#include <Edje.h>
#include <Emusic.h>

#include "enna_event.h"
#include "enna_mediaplayer.h"
#include "enna_miniplayer.h"
#include "enna_config.h"
#include "enna_metadata.h"
#include "enna_mplayer.h"
#include "enna_event.h"

#define ENNA_MEDIAPLAYER_STATE_PAUSE 0
#define ENNA_MEDIAPLAYER_STATE_PLAYING 1
#define ENNA_MEDIAPLAYER_STATE_STOP 2

typedef struct _E_Smart_Data E_Smart_Data;


#define ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC 0
#define ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO 1
#define ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_PHOTO 2
#define ENNA_MEDIAPLAYER_PLAYLIST_PLAYLIST 3
  
struct _E_Smart_Data
  {
    Evas_Coord        x, y, w, h;
    Evas_Object       *obj;
    Evas_Object       *edje;
    Emusic_Controller *controller;
    Enna_Mplayer      *mplayer;
    Evas_Object       *cover;
    unsigned int      state;
    unsigned int      playing;
    unsigned int      playlist_type;
    Enna_Playlist     *album_pl;
    Enna_Playlist     *playlist;
    Evas_Object       *miniplayer;
    unsigned int      has_focus;
  };
  


static int new_tags_cb (void* data, int type, void* event);
static int new_file_cb (void* data,int type, void* event);
static int eos_cb (void* data, int type, void* event );
static int _has_suffix(char* str, Evas_List* patterns);
static void _enna_mediaplayer_smart_init(void);
static void _enna_mediaplayer_smart_add(Evas_Object *obj);
static void _enna_mediaplayer_smart_del(Evas_Object *obj);
static void _enna_mediaplayer_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
static void _enna_mediaplayer_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h);
static void _enna_mediaplayer_smart_show(Evas_Object *obj);
static void _enna_mediaplayer_smart_hide(Evas_Object *obj);
static void _enna_mediaplayer_smart_color_set(Evas_Object *obj, int r, int g, int b, int a);
static void _enna_mediaplayer_smart_clip_set(Evas_Object *obj, Evas_Object *clip);
static void _enna_mediaplayer_smart_clip_unset(Evas_Object *obj);

/* local subsystem globals */
static Evas_Smart *_e_smart = NULL;
//static int first_played = 0;
/* externally accessible functions */
EAPI Evas_Object *
enna_mediaplayer_add(Evas *evas)
{
  _enna_mediaplayer_smart_init();
  return evas_object_smart_add(evas, _e_smart);
}

static int 
_video_exit_cb(void *data, int type, void *event)
{
  E_Smart_Data		*sd;
  
  sd = (E_Smart_Data*) data;
  
  enna_mplayer_exited(sd->mplayer);
  sd->has_focus = 0;

  return 1;
}

EAPI int
enna_mediaplayer_play(Evas_Object* obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  if (!sd)
    return 0;

  edje_object_signal_emit(sd->edje, "play", "");
  enna_miniplayer_infos_set(sd->miniplayer, strdup(_("Unknown")), strdup(_("Unknown")), strdup(_("Unknown")), NULL);

  if (sd->playing == ENNA_MEDIAPLAYER_STATE_PLAYING)
    {

  if ( sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC)
    emusic_controller_play(sd->controller);
  else if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)
    {
      Enna_Playlist_Element *element;
      ecore_event_handler_add(ECORE_EXE_EVENT_DEL, _video_exit_cb, sd);
      dbg("selected : %d\n", sd->album_pl->selected);
      element = ecore_list_goto_index(sd->album_pl->elements, sd->album_pl->selected);
      sd->has_focus = 1;
      enna_mplayer_run(sd->mplayer, element->filename);
    }
    }
  return 1;
  
}

EAPI int
enna_mediaplayer_pause(Evas_Object* obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return 0;

  edje_object_signal_emit(sd->edje, "pause", "");
  sd->playing = ENNA_MEDIAPLAYER_STATE_PAUSE;
  //dbg("pause\n");
  return emusic_controller_pause(sd->controller);

}

EAPI int 
enna_mediaplayer_playlist_play(Evas_Object *obj)
{
  E_Smart_Data *sd;
  Enna *enna;
  Enna_Playlist_Element *element;
  sd = evas_object_smart_data_get(obj);
  
  if (!sd)
    return 0;
  
  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!enna)
    return 0;

  emusic_controller_stop(sd->controller);
  //dbg("detroy playlist\n");
  emusic_controller_playlist_destroy(sd->controller, "playlist");
  emusic_controller_playlist_new(sd->controller, "playlist");
  ecore_list_goto_first(sd->playlist->elements);
  while( (element = ecore_list_next(sd->playlist->elements)) )
    emusic_controller_playlist_add(sd->controller, element->filename, "playlist");
  emusic_controller_playlist_goto_id(sd->controller, sd->playlist->selected);
  //dbg("selected : %d\n", sd->playlist->selected);
  enna_mediaplayer_play(enna->mediaplayer);
  
  return 1;
  
}

EAPI int 
enna_mediaplayer_next(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  
  if ( sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC)
    emusic_controller_next(sd->controller);
  else if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)
    {      
      enna_mplayer_process_event(sd->mplayer, enna_event_stop);
      sd->album_pl->selected++;
      if (sd->album_pl->selected >= sd->album_pl->nb_elements)
        sd->album_pl->selected = sd->album_pl->nb_elements;
      else
        enna_mediaplayer_play(obj);
     
    }
  return 1;

}

EAPI int 
enna_mediaplayer_prev(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  
  if ( sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC)
    emusic_controller_prev(sd->controller);
  else if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)
    {      
      enna_mplayer_process_event(sd->mplayer, enna_event_stop);
      sd->album_pl->selected--;
      if (sd->album_pl->selected < 0)
        sd->album_pl->selected = 0;
      else
        enna_mediaplayer_play(obj);
     
    }
  return 1;

}

EAPI int 
enna_mediaplayer_stop(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  if ( sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC)
    emusic_controller_stop(sd->controller);
  else if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)      
    enna_mplayer_process_event(sd->mplayer, enna_event_stop);

  sd->has_focus = 0;
  return 1;
}

EAPI int 
enna_mediaplayer_ffw(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)      
    enna_mplayer_process_event(sd->mplayer, enna_event_fastforward);

  return 1;
}

EAPI int 
enna_mediaplayer_rew(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  if  (sd->playlist_type == ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO)      
    enna_mplayer_process_event(sd->mplayer, enna_event_rewind);

  return 1;
}

EAPI int 
enna_mediaplayer_ratio(Evas_Object *obj)
{
  return 0;
}

EAPI int
enna_mediaplayer_has_focus_get(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);

  return sd->has_focus;

}

EAPI int 
enna_mediaplayer_playlist_goto_id(Evas_Object *obj, unsigned int id)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd || sd->controller)
    return 0;

  return emusic_controller_playlist_goto_id(sd->controller, id);
}

EAPI int
enna_mediaplayer_playlist_add(Evas_Object* obj, char* filename)
{
  E_Smart_Data *sd;
  unsigned type = ENNA_MEDIA_MUSIC;
  Enna_Playlist_Element *element;
  sd = evas_object_smart_data_get(obj);
  if (!sd || !sd->playlist)
    return 0;

  element = (Enna_Playlist_Element*)malloc(sizeof(Enna_Playlist_Element));
  
  if (_has_suffix(filename, enna_config_extensions_get("music")))
      type = ENNA_MEDIA_MUSIC;
  else if (_has_suffix(filename, enna_config_extensions_get("video")))
      type = ENNA_MEDIA_VIDEO;
  else if (_has_suffix(filename, enna_config_extensions_get("photo")))
      type = ENNA_MEDIA_PHOTO;
  
  element->media_type = type;
  element->filename = strdup(filename);
  
  ecore_list_append(sd->playlist->elements, element);
  
  sd->playlist->nb_elements++;
  //dbg("append %s %d\n", element->filename, sd->playlist->nb_elements);
  return 1;
}

EAPI Enna_Playlist *
enna_mediaplayer_playlist_get(Evas_Object* obj, char *name)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  
  if (!sd || !sd->playlist)
    return NULL;
  
  return sd->playlist;
}

EAPI void 
enna_mediaplayer_playlist_destroy(Evas_Object *obj, char *name)
{
}

EAPI void
enna_mediaplayer_cdda_add(Evas_Object *obj, char *device, int nb_tracks, int selected)
{
  E_Smart_Data *sd;
  int i = 0;
  Enna *enna;
  
  sd = evas_object_smart_data_get(obj);
  if (!sd || !device)
    return;
  
  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!enna)
    return;
  
  
  if (sd->album_pl)
    ecore_list_destroy(sd->album_pl->elements);

  sd->album_pl->nb_elements = 0;
  sd->album_pl->duration = 0;
  sd->album_pl->size = 0;
  sd->album_pl->elements = ecore_list_new();
  //dbg("nbtracks ! %d\n", nb_tracks);
  for (i = 0; i < nb_tracks; i++)
  {
    char preffix[4096];
    Enna_Playlist_Element *element;

    element = (Enna_Playlist_Element*)malloc(sizeof(Enna_Playlist_Element));
    element->media_type = ENNA_MEDIA_MUSIC;
    //dbg("%d\n", bcl);
    sprintf(preffix, "cdda://%d", i+1);
    element->filename = strdup(preffix);
    //dbg("%s\n", preffix);
    ecore_list_append(sd->album_pl->elements, element);
  }

  sd->album_pl->nb_elements = nb_tracks;
  sd->album_pl->selected = selected-1;
  //dbg("file selected ! %d\n", selected);
  ecore_list_goto_index(sd->album_pl->elements, selected-1);

  {
    Enna_Playlist_Element *element;
    emusic_controller_stop(sd->controller);
    //dbg("detroy playlist\n");
    emusic_controller_playlist_destroy(sd->controller, "playlist");
    emusic_controller_playlist_new(sd->controller, "playlist");
    ecore_list_goto_first(sd->album_pl->elements);
    while( (element = ecore_list_next(sd->album_pl->elements)) )
      emusic_controller_playlist_add(sd->controller, element->filename, "playlist");
    emusic_controller_playlist_goto_id(sd->controller, sd->album_pl->selected);
    //dbg("selected : %d\n", sd->album_pl->selected);
  }
  enna_mediaplayer_play(enna->mediaplayer);
}

EAPI void 
enna_mediaplayer_album_add(Evas_Object *obj, char *directory, char *filename, char *filter)
{
  E_Smart_Data *sd;
  Ecore_List *files;
  char *file;
  int i = 0;
  int id = 0;
  Enna *enna;
  
  sd = evas_object_smart_data_get(obj);
  if (!sd || !directory || !filename)
    return;
  
  dbg("album add\n");

  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!enna)
    return;  
  
  if (sd->album_pl)
    ecore_list_destroy(sd->album_pl->elements);
  
  sd->album_pl->nb_elements = 0;
  sd->album_pl->duration = 0;
  sd->album_pl->size = 0;
  sd->album_pl->elements = ecore_list_new();
  dbg("directory : %s\n", directory);
  files = ecore_file_ls (directory);
  file = ecore_list_goto_first (files);
  while ((file = (char *) ecore_list_next (files)) != NULL)
    {
      if (_has_suffix(file, enna_config_extensions_get(filter)))
        {
          char full_filename[4096];
          Enna_Playlist_Element *element;

          element = (Enna_Playlist_Element*)malloc(sizeof(Enna_Playlist_Element));

          if (!strcmp(filter, "music"))
            element->media_type = ENNA_MEDIA_MUSIC;
          else if (!strcmp(filter, "video"))
            element->media_type = ENNA_MEDIA_VIDEO;
          else if (!strcmp(filter, "photo"))
            element->media_type = ENNA_MEDIA_PHOTO;

          sprintf(full_filename, "%s/%s", directory, (char*)file);
          element->filename = strdup(full_filename);
          ecore_list_append(sd->album_pl->elements, element);
          dbg("add : %s\n", full_filename);
          if (!strcmp(full_filename, filename))
            id = i;
          i++;
        }
    }
  
  sd->album_pl->nb_elements = i;
  sd->album_pl->selected = id;
  //dbg("file selected ! %d\n", id);
  ecore_list_goto_index(sd->album_pl->elements, id);

  enna_mediaplayer_stop(obj);

  if (!strcmp(filter, "music"))
  {
    Enna_Playlist_Element *element;

    sd->playlist_type = ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_MUSIC;
    emusic_controller_stop(sd->controller);
    emusic_controller_playlist_destroy(sd->controller, "playlist");
    emusic_controller_playlist_new(sd->controller, "playlist");
    ecore_list_goto_first(sd->album_pl->elements);
    while( (element = ecore_list_next(sd->album_pl->elements)) )
      {
        char tmp[4096];
        sprintf(tmp, "file://%s", element->filename);
        emusic_controller_playlist_add(sd->controller, tmp, "playlist");
      }
    emusic_controller_playlist_goto_id(sd->controller, sd->album_pl->selected);
    //dbg("selected : %d\n", sd->album_pl->selected);
  }
  else
    {
      sd->playlist_type = ENNA_MEDIAPLAYER_PLAYLIST_ALBUM_VIDEO;
    }

  enna_mediaplayer_play(enna->mediaplayer);
}

EAPI void 
enna_mediaplayer_radio_add(Evas_Object *obj, char *directory, char *filename, char *filter)
{
  E_Smart_Data *sd;
  Enna *enna;
  
  sd = evas_object_smart_data_get(obj);
  if (!sd || !directory || !filename)
    return;
  
  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!enna)
    return;
  
  
  if (sd->album_pl)
    ecore_list_destroy(sd->album_pl->elements);
  
  sd->album_pl->nb_elements = 0;
  sd->album_pl->duration = 0;
  sd->album_pl->size = 0;
  sd->album_pl->elements = ecore_list_new();
  
  //dbg("fileter : %s\n", filter);
  if (_has_suffix(filename, enna_config_extensions_get(filter)))
    {
      Enna_Playlist_Element *element;
      element = (Enna_Playlist_Element*)malloc(sizeof(Enna_Playlist_Element));
      element->media_type = ENNA_MEDIA_MUSIC;
      element->filename = strdup(filename);
      //dbg("add radio : %s\n", filename);
      ecore_list_append(sd->album_pl->elements, element);
    }

  
    
  sd->album_pl->nb_elements = 0;
  sd->album_pl->selected = 0;
  if (!strcmp(filter, "radio"))
  {
    Enna_Playlist_Element *element;
    element =  ecore_list_goto_first(sd->album_pl->elements);
    emusic_controller_stop(sd->controller);
    //dbg("detroy playlist\n");
    emusic_controller_playlist_destroy(sd->controller, "playlist");
    emusic_controller_playlist_new(sd->controller, "playlist");
    //dbg("load radio : %s\n", element->filename);
    emusic_controller_playlist_load(sd->controller, element->filename);

  }
  enna_mediaplayer_play(enna->mediaplayer);
}

EAPI void enna_mediaplayer_album_play(Evas_Object *obj)
{
}

EAPI void 
enna_mediaplayer_process_event(Evas_Object *obj, enna_event event)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  
  dbg("mediaplayer event %d\n", event);

  switch (event)
    {
    case enna_event_play:
    case enna_event_pause:
      if (sd->playing == ENNA_MEDIAPLAYER_STATE_PLAYING)
        enna_mediaplayer_pause(obj);
      else
        enna_mediaplayer_play(obj);
      break;
    case enna_event_fastforward:
       enna_mediaplayer_ffw(obj);
      break;
    case enna_event_rewind:
       enna_mediaplayer_rew(obj);
      break;
    case enna_event_prev:
      dbg("Prev\n");
      enna_mediaplayer_prev(obj);
      //emusic_controller_prev(sd->controller);
      break;
      
    case enna_event_next:
      dbg("Next\n");
      enna_mediaplayer_next(obj);
      //emusic_controller_next(sd->controller);
      break;
      
    case enna_event_stop:
    case enna_event_exit :
      enna_mediaplayer_stop(obj);
      //emusic_controller_stop(sd->controller);
	break;
        
    default:
      break;
    }
}

EAPI Evas_Object*
enna_mediaplayer_menu_get(Evas_Object *obj)
{

  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return NULL;

  return sd->miniplayer;

  
}

static int 
_position_timer (void *data)
{
  double pos, len;
  double fraction;
  long ph, pm, ps, lh, lm, ls;
  char buf[256];
  char buf2[256];
  E_Smart_Data *sd;

  sd = data;

  pos = emusic_controller_position_get(sd->controller);
  len = emusic_controller_length_get(sd->controller);
  lh = len / 3600000;
  lm = len / 60 - (lh * 60);
  ls = len - (lm * 60);
  ph = pos / 3600;
  pm = pos / 60 - (ph * 60);
  ps = pos - (pm * 60);
  snprintf(buf, sizeof(buf), "%li:%02li:%02li",
           ph, pm, ps);
  snprintf(buf2, sizeof(buf2), "%li:%02li:%02li",
           lh, lm, ls);
  if (len)
    fraction = pos / len;
  else
    fraction = 0.0;

  edje_object_part_text_set(sd->edje, "enna.text.length",  buf2);
  edje_object_part_text_set(sd->edje, "enna.text.position",  buf);
  edje_object_part_drag_value_set(sd->edje, "enna.drag.progress", fraction, 0.0);

  return 1;
}

char* dirname(char *file)
{
  char* s;
  char* path;
  path = strdup(file);

  s=path+strlen(path)-1;
  while (s && *s == '/')
    {
      *s = '\0';
      s=path+strlen(path)-1;
    }
  s = strrchr(path, '/');
  if (s && *s)
    *s = '\0';

  if (ecore_str_has_prefix(file, "file://"))
    path = &path[7];

  return path;
}


char* _cover_set(char *artist, char *album, char *filename)
{
  /*char* cover_filename;*/
  Ecore_List* files;
  char* list_item;
  char dir[1024];
  char* path;

  path = dirname(filename);


  sprintf(dir, "%s/%s.jpg", path, album);
  if (ecore_file_exists(dir))
    return strdup(dir);

  files = ecore_file_ls(path);
  list_item = ecore_list_goto_first(files);
  while((list_item = (char*)ecore_list_next(files)) != NULL)
    {
      sprintf(dir, "%s/%s",path,list_item);
      if (!ecore_file_is_dir(dir))
        {
          if (ecore_str_has_suffix(dir, ".png") || ecore_str_has_suffix(dir,".jpg"))
            return strdup(dir);
        }
    }

  //cover_filename = enna_cover_get(artist, album, path);
  /*if (cover_filename)
    {
      dbg("Cover file find on Amazon : %s\n", cover_filename);
      return cover_filename;
    }
  else
    dbg("Can't get cover on amazon !\n");
  */
  return NULL;
}

static int new_tags_cb (void* data,int type, void* event)
{
  /* Emusic_Metadata* metadata;
  E_Smart_Data *sd;
  char* cover_filename;
  Evas_Coord w, h;
  
  metadata = (Emusic_Metadata*) event;
  sd = data;

  edje_object_part_text_set(sd->edje, "enna.text.album",metadata->album);
  edje_object_part_text_set(sd->edje, "enna.text.title", metadata->title);
  edje_object_part_text_set(sd->edje, "enna.text.artist", metadata->artist);

  cover_filename = _cover_set(metadata->artist, metadata->album, metadata->uri);
  if (cover_filename)
    {
      edje_object_signal_emit(sd->edje, "enna,cover,set", "enna");
      edje_object_part_swallow(sd->edje, "enna.swallow.cover", sd->cover);
      edje_object_part_geometry_get(sd->edje, "enna.swallow.cover", NULL, NULL, &w, &h);
      evas_object_image_file_set(sd->cover, cover_filename, NULL);
      evas_object_resize(sd->cover, w, h);
      evas_object_image_fill_set(sd->cover, 0, 0, w, h);
      evas_object_show(sd->cover);
    }
  else
    edje_object_signal_emit(sd->edje, "enna,cover,unset", "enna");

  enna_miniplayer_infos_set(sd->miniplayer, metadata->title, metadata->album, metadata->artist, cover_filename);
  edje_object_signal_emit(sd->edje, "play", "");
  */
  return 1;
}

static int
new_file_cb (void* data,int type, void* event)
{
  Enna_Metadata* metadata;
  char *filename;
  E_Smart_Data *sd;
  Evas_Coord w, h;

  sd = data;
  filename = (char*)event;
  if (ecore_str_has_prefix(filename, "file://"))
    filename = filename+7;
  dbg("new file played : %s\n", filename);

  metadata = enna_metadata_get(filename);

  edje_object_part_text_set(sd->edje, "enna.text.album",metadata->album);
  edje_object_part_text_set(sd->edje, "enna.text.title", metadata->title);
  edje_object_part_text_set(sd->edje, "enna.text.artist", metadata->artist);

  if (metadata->cover_filename)
    {
      edje_object_signal_emit(sd->edje, "enna,cover,set", "enna");
      edje_object_part_swallow(sd->edje, "enna.swallow.cover", sd->cover);
      edje_object_part_geometry_get(sd->edje, "enna.swallow.cover", NULL, NULL, &w, &h);
      evas_object_image_file_set(sd->cover, metadata->cover_filename, NULL);
      evas_object_resize(sd->cover, w, h);
      evas_object_image_fill_set(sd->cover, 0, 0, w, h);
      evas_object_show(sd->cover);
    }
  else
    edje_object_signal_emit(sd->edje, "enna,cover,unset", "enna");

  enna_miniplayer_infos_set(sd->miniplayer, metadata->title, metadata->album, metadata->artist, metadata->cover_filename);

  /*
  dbg("--\n");
  dbg("%s\n", metadata->title ? metadata->title : "Unknown" );
  dbg("%s\n", metadata->album ? metadata->album : "Unknown");
  dbg("%s\n", metadata->artist ? metadata->artist : "Unknown");
  dbg("%s\n", metadata->genre ? metadata->genre : "Unknown");
  dbg("%s\n", metadata->filetype ? metadata->filetype : "Unknown");
  dbg("%s\n", metadata->comment ? metadata->comment : "Unknown");
  dbg("%d\n", metadata->id);
  dbg("%d MB\n", metadata->size / (1024 * 1024));
  dbg("%d seconds\n", metadata->duration);
  dbg("%d\n", metadata->cds);
  dbg("%d\n", metadata->cd_nbr);
  dbg("%d\n", metadata->track_nbr);
  dbg("%d\n", metadata->tracks);
  dbg("%d\n", metadata->bitrate);
  dbg("%d\n", metadata->samplerate);
  dbg("%d\n", metadata->year);
  dbg("%d\n", metadata->play_count);
  dbg("%d\n", metadata->channels);
  dbg("%s\n", metadata->cover_filename);

  dbg("--\n");*/
  return 1;
}

static int 
eos_cb (void* data, int type, void* event)
{
  return 1;
}

static int
_has_suffix(char* str, Evas_List* patterns)
{
  Evas_List *l;
  int result = 0;

  if (!patterns || !str || !str[0])
    return 0;
  for (l = patterns; l; l = evas_list_next(l))
    {
      //dbg("pattern : %s\n", (char*)l->data);
      result |= ecore_str_has_suffix(str, (char*)l->data);
    }
  return result;
}

static void
_enna_mediaplayer_smart_init(void)
{
  if (_e_smart)
    return;



  _e_smart = evas_smart_new("enna_mediaplayer",
                            _enna_mediaplayer_smart_add,
                            _enna_mediaplayer_smart_del,
                            NULL, NULL, NULL, NULL, NULL,
                            _enna_mediaplayer_smart_move,
                            _enna_mediaplayer_smart_resize,
                            _enna_mediaplayer_smart_show,
                            _enna_mediaplayer_smart_hide,
                            _enna_mediaplayer_smart_color_set,
                            _enna_mediaplayer_smart_clip_set,
                            _enna_mediaplayer_smart_clip_unset,
                            NULL);
  if (!emusic_init("emusic_gstreamer_plugin"))
    dbg("Error: can't initialize emusic with gstreamer backend !\n");
}

static void
_enna_mediaplayer_smart_add(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = calloc(1, sizeof(E_Smart_Data));
  if (!sd)
    return;
  sd->obj = obj;
  sd->x = 0;
  sd->y = 0;
  sd->w = 0;
  sd->h = 0;
  sd->playing = ENNA_MEDIAPLAYER_STATE_STOP;
  sd->edje =  edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(sd->edje, enna_config_theme_get(), "enna/mediaplayer");

  sd->cover =  evas_object_image_add(evas_object_evas_get(obj));
  edje_object_part_swallow(sd->edje, "enna.swallow.cover", sd->cover);

  /* Emusic Init */
  sd->controller = emusic_controller_new("Enna_MusicPlayer");
  if ( ! emusic_controller_mode_set(sd->controller, EMUSIC_CONTROLLER_MODE_PLAYER) )
    {
      dbg("ERROR: Can't set the player mode to controller %s.\n", sd->controller->name);
      return;
    }

  if ( ! emusic_controller_playlist_new(sd->controller, "playlist") )
    dbg("ERROR: Can't create a new playlist for %s.\n", sd->controller->name);
  
  emusic_controller_volume_set(sd->controller, 0.1);
  emusic_controller_callback_add( sd->controller, "eos", eos_cb, sd);
  emusic_controller_callback_add( sd->controller, "new_tags", new_tags_cb, sd);
  emusic_controller_callback_add( sd->controller, "new_file", new_file_cb, sd);
  ecore_timer_add(0.5, _position_timer, sd);
  evas_object_smart_data_set(obj, sd);
  evas_object_smart_member_add(sd->edje, obj);
  
  /* Init Playlists */
  sd->album_pl = (Enna_Playlist*) malloc(sizeof(Enna_Playlist));
  sd->album_pl->name = strdup("album_pl");
  sd->album_pl->nb_elements = 0;
  sd->album_pl->selected = -1;
  sd->album_pl->size = 0;
  sd->album_pl->duration = 0;
  sd->album_pl->elements = ecore_list_new();
  
  sd->playlist = (Enna_Playlist*) malloc(sizeof(Enna_Playlist));
  sd->playlist->name = strdup("playlist");
  sd->playlist->nb_elements = 0;
  sd->playlist->selected = -1;
  sd->playlist->size = 0;
  sd->playlist->duration = 0;
  sd->playlist->elements = ecore_list_new();

  /* Mplayer */
  sd->mplayer = enna_mplayer_init();

  /* Focus */
  sd->has_focus = 0;

  /* Create miniplayer */
  sd->miniplayer = enna_miniplayer_add(evas_object_evas_get(obj));
  evas_object_show(sd->miniplayer);

}

static void
_enna_mediaplayer_smart_del(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  evas_object_del(sd->edje);
  free(sd);
}

static void
_enna_mediaplayer_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  if ((x == sd->x) && (y == sd->y))
    return;
  {
    Evas_Coord dx, dy;
    Evas_Coord ox, oy;

    dx = x - sd->x;
    dy = y - sd->y;
    if (sd->edje)
      {
        evas_object_geometry_get(sd->edje, &ox, &oy, NULL, NULL);
        evas_object_move(sd->edje, ox + dx, oy + dy);
      }
  }
  sd->x = x;
  sd->y = y;
}

static void
_enna_mediaplayer_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
  E_Smart_Data *sd;
  Evas_Coord cw, ch;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  if ((w == sd->w) && (h == sd->h))
    return;
  sd->w = w;
  sd->h = h;
  if (sd->edje)
    evas_object_resize(sd->edje, w, h);

  if (sd->cover)
    {
      edje_object_part_geometry_get(sd->edje, "enna.swallow.cover", NULL, NULL, &cw, &ch);
      evas_object_resize(sd->cover, cw, ch);
      evas_object_image_fill_set(sd->cover, 0, 0, cw, ch);
    }
  //_enna_mediaplayer_smart_reconfigure(sd);
}

static void
_enna_mediaplayer_smart_show(Evas_Object *obj)
{
  E_Smart_Data *sd;
  //Enna *enna;
  int *nothing;

  nothing = malloc(sizeof(int));
  *nothing = 0x55;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  //enna = evas_data_attach_get(evas_object_evas_get(sd->edje));

  /*  if(enna)
    ecore_event_add(enna->events[ENNA_EVENT_MINIPLAYER_HIDE], nothing, NULL, NULL);
  */
  evas_object_show(sd->edje);

}

static void
_enna_mediaplayer_smart_hide(Evas_Object *obj)
{
  E_Smart_Data *sd;
  //Enna *enna;
  int *nothing;

  nothing = malloc(sizeof(int));
  *nothing = 0x55;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;

  /* enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if(enna)
    ecore_event_add(enna->events[ENNA_EVENT_MINIPLAYER_SHOW], nothing, NULL, NULL);
  */
  evas_object_hide(sd->edje);
  
}

static void
_enna_mediaplayer_smart_color_set(Evas_Object *obj, int r, int g, int b, int a)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  evas_object_color_set(sd->edje, r, g, b, a);
}

static void
_enna_mediaplayer_smart_clip_set(Evas_Object *obj, Evas_Object *clip)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  evas_object_clip_set(sd->edje, clip);
}

static void
_enna_mediaplayer_smart_clip_unset(Evas_Object *obj)
{
  E_Smart_Data *sd;

  sd = evas_object_smart_data_get(obj);
  if (!sd)
    return;
  evas_object_clip_unset(sd->edje);
}
