#include "enna_metadata.h"

#ifndef PATH_MAX
#define PATH_MAX 4096
#endif


typedef struct _File_Data File_Data;
typedef struct _Hash_Data Hash_Data;


struct _File_Data
{
   char *sha;
};

struct _Hash_Data
{
  char *name;
  Evas_Hash *sha_hash;
  Evas_Hash *file_hash;
};

static Eet_Data_Descriptor *edd;
static Eet_Data_Descriptor *edd_hash_sha;
static Eet_Data_Descriptor *edd_hash_file;
static Evas_Hash *sha_hash = NULL;
static Evas_Hash *file_hash = NULL;
static Eet_File *ef;

#ifdef WORDS_BIGENDIAN
# define SWAP(n) (n)
#else
# define SWAP(n) \
    (((n) << 24) | (((n) & 0xff00) << 8) | (((n) >> 8) & 0xff00) | ((n) >> 24))
#endif

#define f1(X,Y,Z) (Z ^ (X & (Y ^ Z)))
#define f2(X,Y,Z) (X ^ Y ^ Z)
#define f3(X,Y,Z) ((X & Y) | (Z & (X | Y)))

#define rol1(x) (x<<1) | ((x>>31) & 1)
#define rol5(x) ((x<<5) | ((x>>27) & 0x1f))
#define rol30(x) (x<<30) | ((x>>2) & 0x3fffffff)



#define AMAZON_ID "0HMNNC83SA7RW4YG45R2"


static xmlNode* 
_parse_xml(xmlNode *root, char *node_name)
{
  xmlNode *n, *children;

  for (n = root; n; n = n->next)
  {
    if ( ( (n->type == XML_ELEMENT_NODE) && (xmlStrcmp(n->name, (xmlChar*)node_name) ) == 0 ) )
    {
      return n;
    }
    if ((children = _parse_xml(n->children, node_name)))
      return children;
  }

  return NULL;
}

static char* 
_get_cover_file(char* file)
{

  xmlDocPtr doc;
  xmlNode *node;
  xmlNode *root = NULL;
	 	
  doc = xmlParseFile(file);

 	
  if (doc == NULL) 
  {
    printf("Erreur Le fichier n'est pas un format XML valide\n");
    return NULL;
  }
	
  root = xmlDocGetRootElement(doc);
	
  if (!(node = _parse_xml(root, "LargeImage")))
  {
    if (!(node = _parse_xml(root, "MediumImage")))
      node = _parse_xml(root, "SmallImage");
  }
	 
  if (!(node = _parse_xml(node, "URL")))
    return NULL;
  else 
    return strdup((char*)xmlNodeGetContent(node)); 	
}

static char* 
_get_asin(char* file, char* node_name)
{

  xmlDocPtr doc;
  xmlNode *node;
  doc = xmlParseFile(file);
  xmlNode *root = NULL;
 	
  if (doc == NULL) 
    printf("Erreur Le fichier n'est pas un format XML valide\n");

	
  root = xmlDocGetRootElement(doc);
	
  if ((node = _parse_xml(root, node_name)))
  {
    printf("%s  : %s\n", node_name,  xmlNodeGetContent(node));
    return (char*)xmlNodeGetContent(node);
  }
  else
  {
    printf("Impossible de récupérer un ASIN valide dans le fichier XML;\n");
    return NULL;
  }

}


static void 
_build_cover_request(char* amazonid, char* asin, char request[1024])
{
  sprintf(request, "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=%s&Operation=ItemLookup&ItemId=%s&ResponseGroup=Images",
          amazonid, asin);
}

static void 
_build_request(char* amazonid, char* keywords, char request[1024])
{
  sprintf(request, "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&SubscriptionId=%s&Operation=ItemSearch&Keywords=%s&SearchIndex=Music", amazonid, keywords);
}

static CURLcode 
_download_http_file(char* file, char* url)
{
  CURL *curl;
  CURLcode res;
  
  FILE *fp = fopen(file, "w");
  
  /* FIXME : Get permissions on file, if cannot write in directory, try to write in $HOME/.enna/cache */
  if (!fp)
    return 0;
  	
  curl = curl_easy_init();
  if(curl) {
    curl_easy_setopt(curl, CURLOPT_URL, url );
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
    res = curl_easy_perform(curl);

    if (res)
      printf("Impossible de récupérer le fichier\n");
	 	
    curl_easy_cleanup(curl);
    fclose(fp);
    return res;
  }
  else
    return 0;
}



/* This function comes from Eclair source code */
static char *
_add_uri_special_chars(const char *uri)
{
  int uri_length;
  char *new_uri;
  int i, j;

  if (!uri)
    return NULL;

  uri_length = strlen(uri);
  new_uri = malloc(uri_length * 3 + 1);

  for (i = 0, j = 0; i < uri_length; i++, j++)
  {
    if (uri[i] == ' ' || uri[i] == ';'  || uri[i] == '&'  || uri[i] == '='  ||
        uri[i] == '<'  || uri[i] == '>'  || uri[i] == '?'  || uri[i] == '@'  ||
        uri[i] == '\'' || uri[i] == '['  || uri[i] == '#'  || uri[i] == '\\' ||
        uri[i] == '$'  || uri[i] == ']'  || uri[i] == '%'  || uri[i] == '^'  ||
        /*uri[i] == '\\' || uri[i] == 'š'  || uri[i] == '+'  || uri[i] == '{'  ||*/
        uri[i] == ','  || uri[i] == '|'  || uri[i] == '/'  || uri[i] == '}'  ||
        uri[i] == ':'  || uri[i] == '~')
    {
      new_uri[j] = '%';
      sprintf(&new_uri[j + 1], "%2x", uri[i]);
      j += 2;
    }
    else
      new_uri[j] = uri[i];
  }
  new_uri[j] = 0;

  return new_uri;
}

static char* 
dirname(char *file)
{
  char* s;
  char* path;
  path = strdup(file);

  s=path+strlen(path)-1;
  while (s && *s == '/')
    {
      *s = '\0';
      s=path+strlen(path)-1;
    }
  s = strrchr(path, '/');
  if (s && *s)
    *s = '\0';

  if (ecore_str_has_prefix(file, "file://"))
    path = &path[7];

  return path;
}

static void 
metadata_cover_get(char *filename, Enna_Metadata *metadata)
{
  Ecore_List* files;
  char* list_item;
  char request[4096];
  char keywords[4096];
  char *asin;
  char *image;
  char tmp[4096];
  char *path;

  if (!metadata->artist || !metadata->album)
    return;

  path = dirname(filename);
  printf("path : %s\n", path);

  sprintf(tmp, "%s/%s.jpg", path, metadata->album);
  if (ecore_file_exists(tmp))
    {
      metadata->cover_filename= strdup(tmp);
      return;
    }

 
  sprintf(keywords, "%s %s", metadata->artist, metadata->album);
  _build_request(AMAZON_ID, _add_uri_special_chars(keywords), request);
  printf("URL : %s\n",request);
  	
  sprintf(tmp, "%s/%s-%s.xml",path, metadata->artist, metadata->album);    
  _download_http_file(tmp, request);
 	
  asin = _get_asin(tmp, "ASIN");
  if (asin)
  {
    _build_cover_request(AMAZON_ID, asin, request);
    printf("URL : %s\n",request);
  }
  else
    {
      ecore_file_recursive_rm(tmp);
      files = ecore_file_ls(path);
      list_item = ecore_list_goto_first(files);
      while((list_item = (char*)ecore_list_next(files)) != NULL)
        {
          sprintf(tmp, "%s/%s",path,list_item);
          if (!ecore_file_is_dir(tmp))
            {
              if (ecore_str_has_suffix(tmp, ".png") || ecore_str_has_suffix(tmp,".jpg"))
                {
                  metadata->cover_filename = strdup(tmp);
                  return;
                }
            }
        }
      return;
    }
  // XXX remove file

  _download_http_file(tmp, request);
  printf("CRASHHHHH : %s\n", tmp);
  image = _get_cover_file(tmp);
  ecore_file_recursive_rm(tmp);
  printf("remove ! %s\n", tmp);
  if (image)
    {
      printf("%s\n", image);
      sprintf(tmp, "%s/%s.jpg", path, metadata->album);
      _download_http_file(tmp, image);
      printf("file download : %s\n", tmp);
      metadata->cover_filename = strdup(tmp);
    }
}

static void 
sha1_hash(unsigned int *data, unsigned int *hash)
{
	unsigned int word[80];
	unsigned int *W;
	int a = hash[0];
	int b = hash[1];
	int c = hash[2];
	int d = hash[3];
	int e = hash[4];
	int t;
	int TEMP;

	W = (unsigned int *) &word;	


	for (t = 0; t < 16; t++) {
		W[t] = SWAP(data[t]);
	}

	/** Data expansion from 16 to 80 blocks **/
	for (t = 16; t < 80; t++) {
		int x = W[t - 3] ^ W[t - 8] ^ W[t - 14] ^ W[t - 16];
		W[t] = rol1(x);
	}

	/** Main loops **/
	for (t = 0; t < 20; t++) {
		TEMP = rol5(a) + f1(b, c, d) + e + W[t] + 0x5a827999;
		e = d;
		d = c;
		c = rol30(b);
		b = a;
		a = TEMP;
	}
	for (; t < 40; t++) {
		TEMP = rol5(a) + f2(b, c, d) + e + W[t] + 0x6ed9eba1;
		e = d;
		d = c;
		c = rol30(b);
		b = a;
		a = TEMP;
	}
	for (; t < 60; t++) {
		TEMP = rol5(a) + f3(b, c, d) + e + W[t] + 0x8f1bbcdc;
		e = d;
		d = c;
		c = rol30(b);
		b = a;
		a = TEMP;
	}
	for (; t < 80; t++) {
		TEMP = rol5(a) + f2(b, c, d) + e + W[t] + 0xca62c1d6;
		e = d;
		d = c;
		c = rol30(b);
		b = a;
		a = TEMP;
	}
	
	hash[0] += a;
	hash[1] += b;
	hash[2] += c;
	hash[3] += d;
	hash[4] += e;
}

static void 
sha1sum_stream(FILE * fd, unsigned int *hashval)
{
	char buffer[64];
	int length = 0;

	hashval[0] = 0x67452301;
	hashval[1] = 0xefcdab89;
	hashval[2] = 0x98badcfe;
	hashval[3] = 0x10325476;
	hashval[4] = 0xc3d2e1f0;

	while (!feof(fd) && !ferror(fd)) {
		int c = fread(&buffer, 1, 64, fd);
		length += c;
		if (feof(fd) || ferror(fd)) {
			int i;
			for (i = c; i < 61; i++) {
				if (i == c) {
					buffer[i] = 0x80;
				}
				else if (i == 60) {
					/* This ends up being swaped twice */
					((unsigned int *) &buffer)[15] = SWAP(length * 8);
				} else {
					buffer[i] = 0;
				}
			
			}
		}
		sha1_hash((unsigned int *) &buffer, hashval);
	}

	return;
}


static char *
sha1_sum(char *filename)
{
  unsigned int *hashval;
  FILE *stream;
  int i;
  char tmp[41];
  
  hashval = malloc(5 * sizeof(unsigned int));
  stream = fopen(filename, "r");
  if (!stream)
    {
      printf("unable to open %s\n", filename);
      return NULL;
    }
  sha1sum_stream(stream, hashval);
  fclose(stream);

  tmp[0] = '\0';
  for (i = 0; i < 5; i++) {
    snprintf(tmp + strlen(tmp), 41 - strlen(tmp), "%08x", hashval[i]);
    
  }
  printf("hash : %s\n", tmp);
  return strdup(tmp);
}



static Enna_Metadata*
metadata_new()
{
  Enna_Metadata *metadata;
  
  metadata = calloc(1, sizeof(Enna_Metadata));
                   
  return metadata;
}


static int
metadata_fs_get(char *filename, Enna_Metadata *metadata)
{
  struct stat infos;

  stat(filename, &infos);

  metadata->size = infos.st_size;
  
  return 1;

}

static int
metadata_id3tag_get(char *filename, Enna_Metadata *metadata)
{
  TagLib_File *tag_file;
  TagLib_Tag *tag;
  TagLib_AudioProperties *prop;

  if (!filename)
    return 0;

  tag_file = taglib_file_new(filename);
  if (!tag_file)
    return 0;

  tag = taglib_file_tag(tag_file);
  if (tag)
    {
      metadata->title = taglib_tag_title(tag);
      metadata->album = taglib_tag_album(tag);
      metadata->artist = taglib_tag_artist(tag);
      metadata->comment = taglib_tag_comment(tag);
      metadata->genre = taglib_tag_genre(tag);
      metadata->year = taglib_tag_year(tag);
      metadata->track_nbr = taglib_tag_track(tag);
    }
  
  prop = (TagLib_AudioProperties *)taglib_file_audioproperties(tag_file);
  if (prop)
    {
      metadata->duration = taglib_audioproperties_length(prop);
      metadata->bitrate = taglib_audioproperties_bitrate(prop);
      metadata->samplerate = taglib_audioproperties_samplerate(prop);
      metadata->channels = taglib_audioproperties_channels(prop);
    }
  return 1;
}


static Evas_Bool
_hash_print(Evas_Hash *hash, const char *key, void *data, void *fdata)
{
  printf("--\n");
  printf("%s %s\n", key, ((Enna_Metadata *)data)->title ? ((Enna_Metadata *)data)->title : "Unknown" );
  printf("%s %s\n", key, ((Enna_Metadata *)data)->album ? ((Enna_Metadata *)data)->album : "Unknown");
  printf("%s %s\n", key, ((Enna_Metadata *)data)->artist ? ((Enna_Metadata *)data)->artist : "Unknown");
  printf("%s %s\n", key, ((Enna_Metadata *)data)->genre ? ((Enna_Metadata *)data)->genre : "Unknown");
  printf("%s %s\n", key, ((Enna_Metadata *)data)->filetype ? ((Enna_Metadata *)data)->filetype : "Unknown");
  printf("%s %s\n", key, ((Enna_Metadata *)data)->comment ? ((Enna_Metadata *)data)->comment : "Unknown");
  printf("%s %d\n", key, ((Enna_Metadata *)data)->id);
  printf("%s %d MB\n", key, ((Enna_Metadata *)data)->size / (1024 * 1024));
  printf("%s %d seconds\n", key, ((Enna_Metadata *)data)->duration);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->cds);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->cd_nbr);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->track_nbr);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->tracks);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->bitrate);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->samplerate);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->year);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->play_count);
  printf("%s %d\n", key, ((Enna_Metadata *)data)->channels);
  printf("%s %s\n", key, ((Enna_Metadata *)data)->cover_filename);

  printf("--\n");
  return 1;
}

static Eet_Data_Descriptor *
metadata_edd_descriptor_create()
{
  Enna_Metadata metadata;
  File_Data file_data;
  Eet_Data_Descriptor *edd;

  edd_hash_sha = eet_data_descriptor_new("metadata", sizeof(metadata),
                                          (void*)evas_list_next,
                                          (void*)evas_list_append,
                                          (void*)evas_list_data,
                                          (void*)evas_list_free,
                                          (void*)evas_hash_foreach,
                                          (void*)evas_hash_add,
                                          (void*)evas_hash_free);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "title", title, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "artist", artist, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "album", album, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "genre", genre, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "filetype", filetype, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "comment", comment, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "id", id, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "size", size, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "duration", duration, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "cd_nbr", cd_nbr, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "cds", cds, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "track_nbr", track_nbr, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "tracks", tracks, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "bitrate", bitrate, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "samplerate", samplerate, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "year", year, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "play_count", play_count, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "channels", channels, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "id", id, EET_T_INT);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_sha, Enna_Metadata, "cover_filename", cover_filename, EET_T_STRING);

   edd_hash_file = eet_data_descriptor_new("file_data", sizeof(file_data),
                                           (void*)evas_list_next,
                                           (void*)evas_list_append,
                                           (void*)evas_list_data,
                                           (void*)evas_list_free,
                                           (void*)evas_hash_foreach,
                                           (void*)evas_hash_add,
                                           (void*)evas_hash_free);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd_hash_file, File_Data, "sha", sha, EET_T_STRING);

   edd = eet_data_descriptor_new("hash_data", sizeof(Hash_Data),
                                 (void*)evas_list_next,
                                 (void*)evas_list_append,
                                 (void*)evas_list_data,
                                 (void*)evas_list_free,
                                 (void*)evas_hash_foreach,
                                 (void*)evas_hash_add,
                                 (void*)evas_hash_free);
   EET_DATA_DESCRIPTOR_ADD_BASIC(edd, Hash_Data, "name", name, EET_T_STRING);
   EET_DATA_DESCRIPTOR_ADD_HASH (edd, Hash_Data, "sha_hash", sha_hash, edd_hash_sha);
   EET_DATA_DESCRIPTOR_ADD_HASH (edd, Hash_Data, "file_hash", file_hash, edd_hash_file);
   return edd;

}

static Enna_Metadata*
_process_file(char *filename)
{
  char *sha;
  Enna_Metadata *metadata;
  File_Data *file_data;

  /* Lire la table des fichiers a la recherche du filename */
  file_data = evas_hash_find(file_hash, filename);
  if (file_data)
    {
      /* Le filename a été trouvé dans la table des fichier */
      printf("cette clef a été trouvée : %s - %s\n", filename, file_data->sha);
      /* On recherche le SHA1 dans la table des sha */
      metadata = evas_hash_find(sha_hash, file_data->sha); 
      if (metadata)
        {
          printf("metadata trouvé\n");
          return metadata;
        }
      else
        {
          /* 
             probléme : le nom de fichier a été trouvé dans la table, mais le sha correspondant 
             n'a pas de cléf correspondant 
             On récupére les metadata sur le fichier, on calcule le sha, on ajoute les metadata 
             dans la table des sha et on modifie la table des fichiers 
          */
          printf("probléme rencontré : file associé a un sha, mais sha non trouvé \n");
          metadata = metadata_new();  
          metadata_id3tag_get(filename, metadata);
          metadata_fs_get(filename, metadata);
          metadata_cover_get(filename, metadata);
          sha = sha1_sum(filename);
          sha_hash = evas_hash_add(sha_hash, sha, metadata);
          free(file_data->sha);
          file_data->sha = strdup(sha);
          evas_hash_modify(file_hash, filename, file_data);
          return metadata;
        }
    }
  else
    {
      /* le fichier n'existe pas dans la table des fichiers
         on calcule le sha */
      sha = sha1_sum(filename);
      /* le sha existe t'il dans la table */
      metadata = evas_hash_find(sha_hash, sha);
      if (metadata)
        {
          /* On ajoute le nom de fichier correspondant au sha dans la table des fichiers */
          printf("probléme : fichier non présetn dans la table des fichiers mais sha existe\n");
          file_data = malloc(sizeof(File_Data));
          file_data->sha = strdup(sha);
          file_hash = evas_hash_add(file_hash, filename, file_data);
          return metadata;
        }
      else
        {
          /* Aucune des deux entrées n'existe */
          /* on les creés toutes les deux */
          metadata = metadata_new();  
          metadata_id3tag_get(filename, metadata);
          metadata_fs_get(filename, metadata);
          metadata_cover_get(filename, metadata);
          sha = sha1_sum(filename);
          file_data = malloc(sizeof(File_Data));
          file_data->sha = strdup(sha);
          sha_hash = evas_hash_add(sha_hash, sha, metadata);
          file_hash = evas_hash_add(file_hash, filename, file_data);
          return metadata;
        }      
    }
}


static void
metadata_scan_directory(char *path)
{
  Ecore_List *files = NULL;
  char *filename = NULL;
  char dir[PATH_MAX];

  files = ecore_file_ls (path); 
  filename = ecore_list_goto_first (files);
   
  while ((filename = (char *) ecore_list_next (files)) != NULL)
    {
      sprintf (dir, "%s/%s", path, filename);
      if (filename[0] == '.')
        continue; 
      else if (ecore_file_is_dir (dir))
        {
          metadata_scan_directory(strdup(dir));
        }
      else if (ecore_str_has_suffix(filename, ".mp3"))
        {
          //printf("%s\n", dir);
          _process_file(dir);
        }
    }
}

EAPI Enna_Metadata *
enna_metadata_get(char *filename)
{
  Enna_Metadata *metadata;

  metadata = _process_file(filename);
  _hash_print(NULL, "", metadata, NULL);

  return metadata;
}


EAPI void 
enna_metadata_init(void)
{
   Hash_Data *eet_data;
   void *data;
   int size;
   eet_init();
   char tmp[PATH_MAX];

   /* Create eet data descriptor */
   edd = metadata_edd_descriptor_create();
   /* Try to open db.eet */
   
   sprintf(tmp, "%s/.enna", enna_user_home_get());
   if (!ecore_file_is_dir(tmp))
     ecore_file_mkdir(tmp);
   
   sprintf(tmp, "%s/.enna/music_db.eet", enna_user_home_get());

   /* open eet file and decode it */
   ef = eet_open(tmp, EET_FILE_MODE_READ);
   if (!ef)
     {
       printf("Unable to open %s\n", tmp);
     }
   else
     {
       data = eet_read(ef, "data", &size);
       printf("size read : %d\n", size);
       eet_close(ef);
       eet_data = eet_data_descriptor_decode(edd, data, size);
       sha_hash =  eet_data->sha_hash;
       file_hash =  eet_data->file_hash;
     }
}

EAPI void
enna_metadata_shutdown(void)
{
  Hash_Data eet_data;
  void *data;
  int size;
  char tmp[PATH_MAX];
   
  eet_data.name = "Hash";
  eet_data.sha_hash = sha_hash;
  eet_data.file_hash = file_hash;

  data = eet_data_descriptor_encode(edd, &eet_data, &size);
  printf("size to write : %d\n", size);
  sprintf(tmp, "%s/.enna/music_db.eet", enna_user_home_get());
  ef = eet_open(tmp, EET_FILE_MODE_WRITE);
  if (ef)
    {
      eet_write(ef, "data", data, size, 0);            
      eet_close(ef);
    }
  evas_hash_free(sha_hash);
  evas_hash_free(file_hash);
  eet_data_descriptor_free(edd);
  eet_data_descriptor_free(edd_hash_sha);
  eet_data_descriptor_free(edd_hash_file);
  eet_shutdown();
  

}
