/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#include <stdlib.h>
#include <stdio.h>
#include "enna.h"
#include "enna_list.h"
#include "enna_fm.h"
#include "enna_config.h"
#include "enna_volume_manager.h"
#include "enna_mediaplayer.h"
#include "enna_xine.h"

#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
#define INTERNAL_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if (!sd) return;

#define SMART_NAME			"enna_module_video"
#define VIDEO_MODULE_STATE_MENU		0
#define VIDEO_MODULE_STATE_FM		1
#define VIDEO_MODULE_STATE_MPLAYER	2
#define VIDEO_MODULE_STATE_XINE		3

typedef struct _E_Smart_Data E_Smart_Data;

struct _E_Smart_Data
  {
    Evas_Coord   	coord_x, coord_y, coord_w, coord_h;
    Evas_Object   	*edje;
    Evas_Object		*obj;
    Evas_List		*modules;
    Evas_Object		*menu;
    Evas_Object		*fm;
    unsigned char	state;
    unsigned char	old_state;
    char		*video_path;
    Evas_Object         *switcher;
    struct xine		x;
  };

/* local subsystem functions */
static void _update_mainmenu(E_Smart_Data *sd, Enna *enna);

/* local subsystem globals */
static Evas_Smart *_e_smart = NULL;

static void
_switch_objects(E_Smart_Data *sd, Evas_Object *obj1, Evas_Object *obj2)
{
  if (!sd || (!obj1 && !obj2))
    return;

  if ( (obj1 && obj2) )
    {
      edje_object_part_unswallow(sd->switcher, obj1);
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit(sd->switcher, "enna,swallow2,default,now", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow1,state1,now", "enna");
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit(sd->switcher, "enna,swallow1,default", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow2,state2", "enna");
      evas_object_hide(obj1);
    }
  else if ( !obj2 )
    {
      edje_object_part_unswallow(sd->switcher, obj1);
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit(sd->switcher, "enna,swallow2,default,now", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow2,state2", "enna");
    }
  else if ( !obj1 )
    {
      edje_object_signal_emit(sd->switcher, "enna,swallow1,state1,now", "enna");
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit(sd->switcher, "enna,swallow1,default", "enna");
      evas_object_show(obj2);
    }
}

static int _late_clear (void *data)
{
   Evas_Object* obj;

   obj = (Evas_Object*) data;
   evas_object_hide(obj);
   evas_object_del(obj);

   return 0;
}

static void _enna_module_video_init(E_Smart_Data *sd)
{
   sd->video_path = strdup(enna_config_get_conf_value_or_default("video_module", "base_path", "/"));
}


EAPI void enna_module_video_process_event( Evas_Object *obj, enna_event event)
{
   API_ENTRY return;

   if (sd->state == VIDEO_MODULE_STATE_MENU)
     enna_list_process_event(sd->menu, event);
   else if (sd->state == VIDEO_MODULE_STATE_FM)
     enna_fm_process_event(sd->fm, event);
   /* if (sd->state == VIDEO_MODULE_STATE_XINE)
     enna_xine_process_event(&sd->x, event);
   else if (sd->state == VIDEO_MODULE_STATE_MPLAYER)
   enna_mplayer_process_event(&sd->m, event);*/
}

static int _app_exit_cb(void *data, int type, void *event)
{
  /*   E_Smart_Data		*sd;

   sd = (E_Smart_Data*) data;

   sd->has_fullfocus	= 0;
   if (sd->state == VIDEO_MODULE_STATE_XINE)
     enna_xine_exited(&sd->x);
   else if (sd->state == VIDEO_MODULE_STATE_MPLAYER)
     enna_mplayer_exited(&sd->m);

     sd->state = sd->old_state;*/
   return 1;
}

static void _run_mplayer_cb(void *data1, void *data2, char *filename)
{

   E_Smart_Data *sd;
   Enna *enna;

   if (!(sd = (E_Smart_Data*) data1))
     return;

   enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
   dbg("add file\n");
   if (!ecore_file_is_dir(filename))
     enna_mediaplayer_album_add(enna->mediaplayer, ecore_file_get_dir(filename), filename, "video");

   /*
   ecore_event_handler_add(ECORE_EXE_EVENT_DEL, _app_exit_cb, sd);
   enna_run_mplayer(&sd->m, filename);
   sd->old_state	= sd->state;
   sd->state		= VIDEO_MODULE_STATE_MPLAYER;
   sd->has_fullfocus	= 1;
   */
}

static void _run_xine_cb (void *data, void *data2)
{
   E_Smart_Data *sd;
   char *device;

   sd = (E_Smart_Data*) data;
   device = (char*) data2;

   if (!sd || !device)
     return;

   ecore_event_handler_add(ECORE_EXE_EVENT_DEL, _app_exit_cb, sd);
   enna_run_xine(&sd->x);

   sd->old_state	= sd->state;
   sd->state		= VIDEO_MODULE_STATE_XINE;
}

static void _exit_fm_cb(void *data1, void *data2)
{
   E_Smart_Data *sd;
   Enna *enna;

   sd = (E_Smart_Data*) data1;
   if (!sd)
     return;

   enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
   _switch_objects(sd, sd->fm, NULL);
   ecore_timer_add (1.0, _late_clear, sd->fm);

   _update_mainmenu(sd, enna);
   sd->state =VIDEO_MODULE_STATE_MENU;
}

static void _watch_videos_cb (void *data, void *data2)
{
   E_Smart_Data *sd;
   char *root_path;
   sd = (E_Smart_Data*) data;
   Enna *enna;

   enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
   if (!enna)
     return;

   root_path =(char*)data2;

   //dbg("Watch video cb\n");

   sd->fm = enna_fm_add(evas_object_evas_get(sd->edje));
   evas_object_show(sd->fm);
   evas_object_hide(sd->menu);
   _switch_objects(sd, sd->menu, sd->fm);
   ecore_timer_add (1.0, _late_clear, sd->menu);
   sd->state = VIDEO_MODULE_STATE_FM;
   enna_fm_filter_set(sd->fm, "video");
   enna_fm_root_set(sd->fm, root_path);
   enna_fm_exe_file_cb_set( sd->fm, _run_mplayer_cb, sd, NULL);
   enna_fm_exit_cb_set (sd->fm, _exit_fm_cb, sd, NULL);
}

static void
_e_smart_reconfigure(E_Smart_Data *sd)
{
  evas_object_move(sd->edje, sd->coord_x, sd->coord_y);
  evas_object_resize(sd->edje, sd->coord_w, sd->coord_h);
}

  /*sd->menu = enna_list_add(evas_object_evas_get(obj));
  icon = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(icon, enna_config_theme_get(), "icon_cdda");
  evas_object_show(icon);
  //enna_list_icon_size_set(sd->menu, 48, 48);
  enna_list_append(sd->menu, icon,  "Watch DVD", 0, NULL, NULL, NULL, NULL);
  icon = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(icon, enna_config_theme_get(), "icon_hd");
  evas_object_show(icon);
  enna_list_append(sd->menu, icon,  "Watch Videos", 0, _watch_videos_cb, NULL, sd, NULL);
  enna_list_selected_set(sd->menu, 0);
  edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->menu);*/

static void
_update_mainmenu(E_Smart_Data *sd, Enna *enna)
{
  Evas_Object *icon;

  //Device *dev;
  Volume *v;


  evas_object_del(sd->menu);
  sd->menu = enna_list_add(evas_object_evas_get(sd->obj));
  //dbg("updatemenu\n");
  //    dev = ecore_list_goto_first(enna->devices_list);
  v = ecore_list_goto_first(enna->volumes);
  //  if (!v) dbg("enna->volumes NULL !!! \n");
  while ((v = ecore_list_next(enna->volumes)))
    {
      //dbg("%s with type : %d\n", v->udi, v->type);
      switch(v->type)
        {
        case DEV_VIDEO_DVD :
          {
            //char tmp[4096];
            //sprintf(tmp, "Listen to Audio CD (%s)", dev->volume_label);
            //dbg("DEV VIDEO_DVD\n");
            icon = edje_object_add(evas_object_evas_get(sd->obj));
            edje_object_file_set(icon, enna_config_theme_get(), 
                                 "icon_dvdvideo");
            //evas_object_show(icon);
            enna_list_append(sd->menu, icon,  "Watch DVD", 
                             0, _run_xine_cb, NULL, sd, v->device);
            break;
          }
        case DEV_DATA_CD :
          {
            //char tmp[4096];
            //sprintf(tmp, "Listen Music on CDROM (%s)", dev->volume_label);
            icon = edje_object_add(evas_object_evas_get(sd->obj));
            edje_object_file_set(icon, enna_config_theme_get(), 
                                 "icon_cdrom");
            //evas_object_show(icon);
            enna_list_append(sd->menu, icon,  "Watch Videos on CDROM", 
                             0, _watch_videos_cb, NULL, sd, v->mount_point);
            break;
          }
        case DEV_DATA_DVD :
          {
            //char tmp[4096];
            //sprintf(tmp, "Listen Music on DVDROM (%s)", dev->volume_label);
            icon = edje_object_add(evas_object_evas_get(sd->obj));
            edje_object_file_set(icon, enna_config_theme_get(), 
                                 "icon_dvdrom");
            //evas_object_show(icon);
            enna_list_append(sd->menu, icon,  "Watch Videos on DVDROM", 
                             0, _watch_videos_cb, NULL, sd, v->mount_point);
            break;
          }
        case DEV_USBDISK :
          {
            //char tmp[4096];
            //sprintf(tmp, "Listen Music on USBDisk (%s)", dev->volume_label);
            icon = edje_object_add(evas_object_evas_get(sd->obj));
            edje_object_file_set(icon, enna_config_theme_get(), 
                                 "icon_usbdisk");
            //evas_object_show(icon);
            enna_list_append(sd->menu, icon,  "Watch Videos on USBDisk", 
                             0, _watch_videos_cb, NULL, sd, v->mount_point);
            break;
          }
        default:
          break;
        }
    }

  
  icon = edje_object_add(evas_object_evas_get(sd->obj));
  edje_object_file_set(icon, enna_config_theme_get(), "icon_hd");
  //evas_object_show(icon);
  enna_list_append(sd->menu, icon,  "Watch Videos", 0, 
                   _watch_videos_cb, NULL, sd, sd->video_path);
  enna_list_selected_set(sd->menu, 0);
  _switch_objects(sd, NULL, sd->menu);

}

static int 
_device_add_event(void *data, int type, void *ev)
{
  E_Smart_Data *sd;
  Enna *enna;


  sd = (E_Smart_Data*) data;

  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));

  //  dbg("dev added\n");

  if (!enna)
    return 0;

  _update_mainmenu(sd, enna);
  return 1;

}

static void
_e_smart_add(Evas_Object *obj)
{
  E_Smart_Data *sd;
  Enna *enna;
  sd = calloc(1, sizeof(E_Smart_Data));
  if (!sd)
    return;

  _enna_module_video_init(sd);

  evas_object_smart_data_set(obj, sd);

  sd->obj = obj;
  sd->coord_x = 0;
  sd->coord_y = 0;
  sd->coord_w = 0;
  sd->coord_h = 0;

  sd->x.xine_exe = NULL;
  sd->x.xine_sd = -1;
  sd->x.xine_handler = NULL;

  sd->state = VIDEO_MODULE_STATE_MENU;
  sd->edje = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(sd->edje, enna_config_theme_get(), "enna/module/video");
  
  if (!sd->edje)
    dbg("NOT EDJE\n");


  sd->switcher = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(sd->switcher, enna_config_theme_get(), "enna/switcher");
  evas_object_smart_member_add(sd->switcher, obj);
  evas_object_show(sd->switcher);
  edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->switcher);


  evas_object_smart_member_add(sd->edje, obj);
  //evas_object_show(sd->edje);
  evas_object_propagate_events_set(obj, 0);

  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!enna)
    return;

  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_VIDEO_DVD_ADD], 
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_DATA_CD_ADD],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_DATA_DVD_ADD],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_USBDISK_ADD],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_CAMERA_ADD],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_VIDEO_DVD_REMOVE], 
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_DATA_CD_REMOVE],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_DATA_DVD_REMOVE],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_USBDISK_REMOVE],
                          _device_add_event, sd);
  ecore_event_handler_add(enna->events[ENNA_EVENT_DEV_CAMERA_REMOVE], 
                          _device_add_event, sd);
  _update_mainmenu(sd, enna);

  evas_object_show(sd->menu);
}

static void
_e_smart_del(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_del(sd->edje);
  free(sd);
}

static void
_e_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
{
  INTERNAL_ENTRY;
  if ((sd->coord_x == x) && (sd->coord_y == y))
    return;
  sd->coord_x = x;
  sd->coord_y = y;
  _e_smart_reconfigure(sd);
}

static void
_e_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
  INTERNAL_ENTRY;

  if ((sd->coord_w == w) && (sd->coord_h == h))
    return;
  sd->coord_w = w;
  sd->coord_h = h;
  _e_smart_reconfigure(sd);

}

static void
_e_smart_show(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_show(sd->edje);
}

static void
_e_smart_hide(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_hide(sd->edje);
}

static void
_e_smart_color_set(Evas_Object *obj, int r, int g, int b, int a)
{
  INTERNAL_ENTRY;
  evas_object_color_set(sd->edje, r, g, b, a);
}

static void
_e_smart_clip_set(Evas_Object *obj, Evas_Object * clip)
{
  INTERNAL_ENTRY;
  evas_object_clip_set(sd->edje, clip);
}

static void
_e_smart_clip_unset(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_clip_unset(sd->edje);
}


/* never need to touch this */
static void _e_smart_init(void)
{
   if (_e_smart)
     return;
   _e_smart = evas_smart_new(SMART_NAME,
	 _e_smart_add,
	 _e_smart_del,
	 NULL, NULL, NULL, NULL, NULL,
	 _e_smart_move,
	 _e_smart_resize,
	 _e_smart_show,
	 _e_smart_hide,
	 _e_smart_color_set,
	 _e_smart_clip_set,
	 _e_smart_clip_unset,
	 NULL);
}

/* externally accessible functions */
EAPI Evas_Object *
enna_module_video_add(Evas *evas)
{
  _e_smart_init();
  return evas_object_smart_add(evas, _e_smart);
}
